@@ -2,7 +2,8 @@
 #' @description Table for coxph.object with model option: TRUE - allow "frailty" or "cluster" model
 #' @param cox.obj.withmodel coxph.object with model option: TRUE
 #' @param dec Decimal point, Default: 2
-#' @return Table, cluster/frailty info, metrics, caption
+#' @param msm Multi state model, Default: NULL
+#' @return Table, cluster/frailty info, metrics, caption 
 #' @details GEE like - cluster, Mixed effect model like - frailty
 #' @examples
 #' library(survival)
@@ -15,19 +16,18 @@
 #' @export
 #' @importFrom survival coxph cluster frailty
 #' @importFrom stats formula update AIC
-
-cox2.display <- function(cox.obj.withmodel, dec = 2) {
+#'
+cox2.display <- function(cox.obj.withmodel, dec = 2, msm = NULL) {
   model <- cox.obj.withmodel
   if (!any(class(model) == "coxph")) {
     stop("Model not from Cox model")
   }
-
   xf <- attr(model$terms, "term.labels") # Independent vars
   xf.old <- xf
   xc <- NULL
   xc.vn <- NULL
+  x.weight <- model$call$weight
   mtype <- "normal"
-
   if (length(grep("strata", xf)) > 0) {
     xf <- xf[-grep("strata", xf)]
   } else if (length(grep("frailty\\(", xf)) > 0) {
@@ -49,7 +49,6 @@ cox2.display <- function(cox.obj.withmodel, dec = 2) {
   formula.surv <- as.character(model$formula)[2]
   formula.ranef <- paste(" + ", xc, sep = "")
   mdata <- model$model
-
   if (length(xc) == 0) {
     formula.ranef <- NULL
   } else {
@@ -62,8 +61,6 @@ cox2.display <- function(cox.obj.withmodel, dec = 2) {
   #  mdata = data.frame(data)
   # }
 
-
-
   if (length(xf) == 1) {
     uni.res <- data.frame(summary(model)$coefficients)
     # uni.res <- data.frame(summary(coxph(as.formula(paste("mdata[, 1]", "~", xf, formula.ranef, sep="")), data = mdata))$coefficients)
@@ -71,58 +68,103 @@ cox2.display <- function(cox.obj.withmodel, dec = 2) {
     names(uni.res)[ncol(uni.res)] <- "p"
     uni.res2 <- NULL
     if (mtype == "normal") {
-      uni.res2 <- uni.res[, c(1, 3, 4, 5)]
-      if (length(grep("robust.se", names(uni.res))) > 0) {
-        uni.res2 <- uni.res[, c(1, 4, 5, 6)]
-      }
+      uni.res2 <- uni.res[, c("z", "p", names(uni.res)[grep("coef", names(uni.res))])]
     } else if (mtype == "cluster") {
-      uni.res2 <- uni.res[, c(1, 4, 5, 6)]
+      uni.res2 <- uni.res[, c("z", "p", names(uni.res)[grep("coef", names(uni.res))])]
     } else {
-      uni.res2 <- uni.res[-nrow(uni.res), c(1, 3, 4, 6)]
+      uni.res2 <- uni.res[, c("p", names(uni.res)[grep("coef", names(uni.res))])]
     }
     fix.all <- coxExp(uni.res2, dec = dec)
     colnames(fix.all) <- c("HR(95%CI)", "P value")
-    # rownames(fix.all) = ifelse(mtype == "frailty", names(model$coefficients)[-length(model$coefficients)], names(model$coefficients))
-    rownames(fix.all) <- names(model$coefficients)
+
+    if (mtype == "frailty") {
+      rownames(fix.all) <- c(names(model$coefficients), "frailty")
+    } else {
+      rownames(fix.all) <- names(model$coefficients)
+    }
   } else {
-    mdata2 <- cbind(matrix(sapply(mdata[, 1], `[[`, 1), ncol = 2), mdata[, -1])
-    names(mdata2)[1:2] <- as.character(model$formula[[2]][2:3])
+    countings <- length(unlist(attr(mdata[[1]], "dimnames")[2]))
+    mdata2 <- cbind(matrix(sapply(mdata[, 1], `[[`, 1), ncol = countings), mdata[, -1])
+    names(mdata2)[1:countings] <- as.character(model$formula[[2]][2:(countings + 1)])
+
+    if (!is.null(x.weight)) {
+      names(mdata2)[ncol(mdata2)] <- as.character(x.weight)
+    }
     if (!is.null(xc.vn)) {
       names(mdata2)[ncol(mdata2)] <- xc.vn
     }
-    basemodel <- update(model, stats::formula(paste(c(". ~ .", xf), collapse = " - ")), data = mdata2)
-    unis <- lapply(xf, function(x) {
-      newfit <- update(basemodel, stats::formula(paste0(". ~ . +", x)), data = mdata2)
-      uni.res <- data.frame(summary(newfit)$coefficients)
-      if (grepl(":", x)) {
-        uni.res <- uni.res[rownames(uni.res) %in% rownames(summary(model)$coefficients), ]
-      } else {
-        uni.res <- uni.res[grep(x, rownames(uni.res)), ]
-      }
-      # uni.res <- uni.res[c(2:nrow(uni.res), 1), ]
-      # uni.res <- data.frame(summary(coxph(as.formula(paste("mdata[, 1]", "~", x, formula.ranef, sep="")), data = mdata))$coefficients)
-      names(uni.res)[ncol(uni.res)] <- "p"
-      uni.res2 <- NULL
-      if (mtype == "normal") {
-        uni.res2 <- uni.res[, c(1, 3, 4, 5)]
-      } else if (mtype == "cluster") {
-        uni.res2 <- uni.res[, c(1, 4, 5, 6)]
-      } else {
-        uni.res2 <- uni.res[, c(1, 3, 4, 6)]
-      }
-      return(uni.res2)
-    })
-    rn.uni <- lapply(unis, rownames)
-    unis2 <- Reduce(rbind, unis)
-    uni.res <- unis2
-    mul.res <- data.frame(coefNA(model))
-    uni.res <- uni.res[rownames(uni.res) %in% rownames(mul.res), ]
-    colnames(mul.res)[ncol(mul.res)] <- "p"
-    fix.all <- cbind(coxExp(uni.res, dec = dec), coxExp(mul.res[rownames(uni.res), names(uni.res)], dec = dec))
-    colnames(fix.all) <- c("crude HR(95%CI)", "crude P value", "adj. HR(95%CI)", "adj. P value")
-    rownames(fix.all) <- rownames(uni.res)
-  }
 
+    if (!is.null(msm)) {
+      baseformula <- stats::formula(paste(c(". ~ .", xf), collapse = " - "))
+      unis <- lapply(xf, function(x) {
+        newfit <- update(model, stats::formula(paste(c(baseformula, x), collapse = "+")))
+        uni.res <- data.frame(summary(newfit)$coefficients)
+        if (grepl(":", x)) {
+          uni.res <- uni.res[rownames(uni.res) %in% rownames(summary(model)$coefficients), ]
+        } else {
+          uni.res <- uni.res[grep(x, rownames(uni.res)), ]
+        }
+        # uni.res <- uni.res[c(2:nrow(uni.res), 1), ]
+        # uni.res <- data.frame(summary(coxph(as.formula(paste("mdata[, 1]", "~", x, formula.ranef, sep="")), data = mdata))$coefficients)
+        names(uni.res)[ncol(uni.res)] <- "p"
+        uni.res2 <- NULL
+        if (mtype == "normal") {
+          uni.res2 <- uni.res[, c("z", "p", names(uni.res)[grep("coef", names(uni.res))])]
+        } else if (mtype == "cluster") {
+          uni.res2 <- uni.res[, c("z", "p", names(uni.res)[grep("coef", names(uni.res))])]
+        } else {
+          uni.res2 <- uni.res[, c("p", names(uni.res)[grep("coef", names(uni.res))])]
+        }
+        return(uni.res2)
+      })
+      rn.uni <- lapply(unis, rownames)
+      unis2 <- Reduce(rbind, unis)
+      uni.res <- unis2
+      mul.res <- data.frame(coefNA(model))
+      uni.res <- uni.res[rownames(uni.res) %in% rownames(mul.res), ]
+      colnames(mul.res)[ncol(mul.res)] <- "p"
+      fix.all <- cbind(coxExp(uni.res, dec = dec), coxExp(mul.res[rownames(uni.res), names(uni.res)], dec = dec))
+      colnames(fix.all) <- c("crude HR(95%CI)", "crude P value", "adj. HR(95%CI)", "adj. P value")
+      rownames(fix.all) <- rownames(uni.res)
+    } else {
+      basemodel <- update(model, stats::formula(paste(c(". ~ .", xf), collapse = " - ")), data = mdata2)
+
+      unis <- lapply(xf, function(x) {
+        newfit <- update(basemodel, stats::formula(paste0(". ~ . +", x)), data = mdata2)
+        uni.res <- data.frame(summary(newfit)$coefficients)
+        if (grepl(":", x)) {
+          uni.res <- uni.res[rownames(uni.res) %in% rownames(summary(model)$coefficients), ]
+        } else {
+          uni.res <- uni.res[grep(x, rownames(uni.res)), ]
+        }
+        # uni.res <- uni.res[c(2:nrow(uni.res), 1), ]
+        # uni.res <- data.frame(summary(coxph(as.formula(paste("mdata[, 1]", "~", x, formula.ranef, sep="")), data = mdata))$coefficients)
+        names(uni.res)[ncol(uni.res)] <- "p"
+        if ("robust.se" %in% names(uni.res)) {
+          uni.res$robust.se <- NULL
+        }
+
+        uni.res2 <- NULL
+        if (mtype == "normal") {
+          uni.res2 <- uni.res[, c("z", "p", names(uni.res)[grep("coef", names(uni.res))])]
+        } else if (mtype == "cluster") {
+          uni.res2 <- uni.res[, c("z", "p", names(uni.res)[grep("coef", names(uni.res))])]
+        } else {
+          uni.res2 <- uni.res[, c("p", names(uni.res)[grep("coef", names(uni.res))])]
+        }
+        return(uni.res2)
+      })
+      rn.uni <- lapply(unis, rownames)
+      unis2 <- Reduce(rbind, unis)
+      uni.res <- unis2
+      mul.res <- data.frame(coefNA(model))
+      uni.res <- uni.res[rownames(uni.res) %in% rownames(mul.res), ]
+      colnames(mul.res)[ncol(mul.res)] <- "p"
+      fix.all <- cbind(coxExp(uni.res, dec = dec), coxExp(mul.res[rownames(uni.res), names(uni.res)], dec = dec))
+      colnames(fix.all) <- c("crude HR(95%CI)", "crude P value", "adj. HR(95%CI)", "adj. P value")
+      rownames(fix.all) <- rownames(uni.res)
+    }
+  }
   ## rownames
   fix.all.list <- lapply(1:length(xf), function(x) {
     fix.all[rownames(fix.all) %in% rn.uni[[x]], ]
@@ -160,7 +202,10 @@ cox2.display <- function(cox.obj.withmodel, dec = 2) {
   if (class(fix.all.unlist)[1] == "character") {
     fix.all.unlist <- t(data.frame(fix.all.unlist))
   }
-  rownames(fix.all.unlist) <- unlist(rn.list)
+  if (is.null(msm)) {
+    rownames(fix.all.unlist) <- unlist(rn.list)
+  }
+
   pv.colnum <- which(colnames(fix.all.unlist) %in% c("P value", "crude P value", "adj. P value"))
   for (i in pv.colnum) {
     fix.all.unlist[, i] <- ifelse(as.numeric(fix.all.unlist[, i]) < 0.001, "< 0.001", round(as.numeric(fix.all.unlist[, i]), dec + 1))
@@ -201,14 +246,19 @@ cox2.display <- function(cox.obj.withmodel, dec = 2) {
 
   ## caption
   surv.string <- as.character(attr(model$terms, "variables")[[2]])
-  time.var.name <- surv.string[2]
-  status.var.name <- surv.string[3]
+  time.var.name <- paste(sapply(2:(length(surv.string) - 1), function(i) paste(surv.string[i])), collapse = ", ")
+  status.var.name <- surv.string[length(surv.string)]
   intro <- paste("Cox model on time ('", time.var.name, "') to event ('", status.var.name, "')", sep = "")
   if (mtype == "cluster") {
     intro <- paste("Marginal", intro, "- Group", cvname)
   } else if (mtype == "frailty") {
     intro <- paste("Frailty", intro, "- Group", cvname)
   }
+  if (!is.null(msm)) {
+    states <- paste(sapply(seq_along(model$states), function(i) paste(i, model$states[i], sep = ": ")), collapse = ", ")
+    intro[2] <- paste("states", states)
+  }
+  states <- model$states
 
   var.names0 <- attr(model$terms, "term.labels")
   if (length(grep("strata", var.names0)) > 0) {
@@ -220,4 +270,5 @@ cox2.display <- function(cox.obj.withmodel, dec = 2) {
   } else {
     return(list(table = fix.all.unlist, ranef = ranef.mat, metric = metric.mat, caption = intro))
   }
-}
+
+}
\ No newline at end of file
